%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% rsort.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [x,w,cdfx,dof]=rsort(x,w);
% sort x in increasing order
% remove multiple entries, and adapt weights w accordingly
% x and w must both be a row or a column
% default input weights are w=1
% 
% if nargout>2, the weighted empirical cdf is computed at x
% dof = length(x) at input is the original number of degrees of freedom



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% snob5.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function x1 = snob5(x,u,v,nreq)
% generates nreq points of class 5 in [u,v]
%
% Input:
% x       the rows are the points already sampled
% u,v     bounds of the box in which the points are to be generated
% nreq    number of points to be generated
%
% Output:
% x1      the rows are the requested points
%         x1 is of dimension nreq x n, where n = size(x,2)



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% snobacc.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function acc = snobacc(x,f,g,G)
% computes a measure of the accuracy of a quadratic model
%
% Input:
% x	x(1,:) is the point around which a quadratic model has been
%	estimated and x(j,:), j=2:size(x,1), are a set of other points
% f	f(j) is the function value pertaining to x(j,:)
% g	coefficients of the quadratic model
% G	matrix proportional to the Hessian
%	the quadratic model is q(y) = g(1) + (y-x0)*g(2:n+1) 
%	+ 0.5*g(n+2)*(y-x0)*G*(y-x0), where x0 = x(j,:), n = size(x,2)
%
% Output:
% acc	estimated accuracy
%	= max(abs(q(x(j,:))-f(j))), j=1:size(x,1) 



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% snobbeta.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function beta = snobbeta(x,jbest,L,beta)
% computes the values beta(j) = norm(L*(x(j,:)-xbest)')^2 for the new
% points 
%
% Input:
% x	the rows are a set of points
% jbest	pointer to the best point
% L 	Cholesky factor from snobhess
% beta	vector containing the beta-values for the old points
%
% Output:
% beta	updated version of beta



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% snobfit.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [request,xbest,fbest,acc] = snobfit(file,x,f,params,dx)
% minimization of a function over a box in R^n
%
% Input:
% file		name of file for input and output
%		if nargin < 5, the program continues a previous run and
%		reads from file.mat
%		the output is (again) stored in file.mat
% x		the rows are a set of new points entering the 
%		optimization algorithm together with their function 
%		values  
% f		matrix containing the corresponding function values
%		and their uncertainties, i.e., f(j,1) = f(x(j,:)) 
% 		and f(j,2) = df(x(j,:))
% params	structure variable defining the box [u,v] in which the
%		points are to be generated, the number nreq of 
%		points to be generated and the probability p that a 
%               point of type 4 is generated
%		params = struct('bounds',{u,v},'nreq',nreq,'p',p)
% dx		only used for the definition of a new problem (when
%		the program should continue from the values stored in
%		file.mat, the call should have only 4 input parameters!)
%	        n-vector of minimal steps, i.e., two points are 
%               considered to be different if they differ by at least 
%               dx(i) in at least one coordinate i
%
% Output:
% request	nreq x (n+3)-matrix (n = size(x,2))
%		request(j,1:n) is the jth newly generated point, 
%		request(j,n+1) is its estimated function value and
%		request(j,n+2) the corresponding estimated standard
%		deviation; request(j,n+3) indicates for which reason
%		the point request(j,1:n) has been generated 
%		request(j,n+3) = 1 best prediction
%		               = 2 explore local valley
%		               = 3 explore alternative valley
%		               = 4 explore empty region
% xbest		current best point
% fbest		current best function value (i.e. function value at 
%		xbest)
% acc		measures the accuracy of the quadratic model at the
%		best point
%		= max(abs(f(x)-q(x)), where the max is taken over the 
%		best point and its n+5 nearest neighbors and q is the 
%		quadratic model
%
% Uses the following m-files (directly or indirectly):
% minq.m and its subprograms
% rsort.m
% snob5.m
% snobacc.m
% snobbeta.m
% snobhess.m
% snobinput.m
% snoblocf.m
% snoblp.m
% snobnan.m
% snobnewb.m
% snobnn.m
% snobpoint.m
% snobrad.m
% snobround.m
% snobscal.m
% snobsplit.m
% snobstd.m
% snobupdt.m



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% snobhess.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [G,sigmaG2,L,beta,g] = snobhess(x,f,xbest,fbest)
% computes a Hessian fit around the point xbest
% f_k-fbest = g'*s_k+0.5*s_k'*G*s_k+eps_k norm(L*s_k)^2, where L is 
% defined as below, s_k = x_k-xbest and sum(eps_k^2) is minimized
%
% Input:
% x		rows are the set of points to be considered
% f		corresponding function values 
% xbest		best point (among the x(j,:))
% fbest		its function value		
%
% Output:
% G		estimate for the Hessian
% sigmaG2	variance of the errors
% L		Cholesky factor of inv(S'*S), where 
%               S = x - ones(K,1)*xbest, K = size(x,1), where the row 
%               with the xbest has been canceled  
% beta		beta(j) = norm(L*(x(j,:)-xbest)')^2
% g		estimate for the gradient



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% snobinput.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [x,f,np,t] = snobinput(x,f)
% checks whether there are any duplicates among the points given by the
% rows of x, throws away the duplicates and computes their average
% function values and an estimated uncertainty
%
% Input:
% x	the rows of x are a set of points
% f	f(j,1) is the function value of x(j,:) and f(j,2) is its
%	uncertainty
%
% Output:
% x	updated version of x (possibly some points have been deleted)
% f	updated version of f (f(j,1) is the average function value and
%	f(j,2) the estimated uncertainty pertaining to x(j,:))  
% np	np(j) is the number of times the row x(j,:) appeared in the
%	input version of x
% t	t(j) is np(j) times the variance of the function values measured
%	for point x(j,:)



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% snoblocf.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [y,f,sigmag2,g] = snoblocf(x0,x,f,xl,xu,G,sigmaG2,beta,u,v)
% computes a local fit around the point x0
% f_k = f + g'*s_k+0.5*gamma*s_k'*G*s_k + eps_k*sqrt(Deltaf_k^2
% +sigmaG2*beta_k)
%
% Input:
% x0		point around which the fit is to be computed
% x		the rows of x are the point x0 (first row) and its
% 		safeguarded nearest neighbors
% f		the corresponding function values and their 
%		uncertainties, i.e., f(j,1) = f(x(j,:)), 
%		f(j,2) = df(x(j,:))
% xl,xu		lower and upper bounds of the box containing x0
% G		estimated Hessian (from snobhess)
% sigmaG2	variance of the errors of the Hessian fit (from 
%		snobhess) 
% beta		beta(j) = norm(L*(x(j,:)-xbest)')^2 (L matrix from 
%		snobhess)
% u,v           box bounds
%
% Output:
% y		best point found by optimizing the local model over
%		the box [xl0,xu0]
% f 		estimated best function value
% sigmag2	variance of the fit
% g		g(1) estimated constant term (f), g(2:n+1) estimated 
%		gradient, g(n+2) estimated coefficient gamma in front
%		of the quadratic term



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% snoblp.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [local,nlocal] = snoblp(f,near,ind)
% computes a pointer to all `local' points (i.e. points whose neighbors
% have lower function values
%
% Input:
% f	f(j) is the function value of point j
% near	near(j,:) contains the indices of the size(near,2) neighbors of
%	point j
% ind	pointer to the boxes to be considered (optional, default
%       1:length(f))
%
% Output:
% local		vector containing the indices of all local points
% nlocal	vector containing the indices of all nonlocal points



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% snobnan.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function f = snobnan(fnan,f,near,inew)
% replaces the function values NaN of a set of points by a value 
% determined by their nearest feasible neighbors, with a safeguard for
% the case that all neighbors are infeasible
%
% Input:
% fnan	  	vector containing the pointers to the points where the
%               function value could not be obtained
% f	  	f(:,1) set of available function values
%		f(:,2) their uncertainty/variation
% near(j,:)	vector pointing to the nearest neighbors of point j
% inew          vector pointing to the new boxes and boxes whose nearest
%               neighbors have changed
%
% Output:
% f		updated version of f



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% snobnewb.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [xl,xu,vol,u,v] = snobnewb(xnew,xl,xu,vol,u,v,u1,v1)
% if xnew contains points outside the box [u,v], the box is made larger
% such that it just contains all new points; moreover, if necessary, 
% [u,v] is made larger to contain [u1,v1]
% the box bounds xl and xu of the boxes on the boundary and the volume 
% measures vol of these boxes are updated if necessary
%
% Input:
% xnew	the rows of xnew contain the new points
% xl	xl(j,:) is the lower bound of box j
% xu	xu(j,:) is the upper bound of box j
% vol	vol(j) is the volume measure of box j
% u,v	old box bounds
% u1,v1 box in which the points are to be generated
%
% Output:
% xl	updated version of xl
% xu 	updated version of xu
% vol	updated version of vol
% u,v	updated box bounds (the old box is contained in the new one)



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% snobnn.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [near,d] = snobnn(x0,x,m)
% computes a safeguarded set of m nearest neighbors to a point x0
% for each coordinate, the nearest point differing from x0 in the ith
% coordinate by at least 1 is chosen
% 
% Input:
% x0	point for which the neighbors are to be found
% x	the rows are the points from which the neighbors are to be
%	chosen (possibly x0 is among these points)
% m	number of neighbors to be found
%
% Output:
% near	vector pointing to the nearest neighbors (i.e. to the rows
%	of x)
% d	maximal distance between x0 and a point of the set of m
% 	nearest neighbors



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%% snobpoint.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function y = snobpoint(x,xl,xu,g,G,u,v)
% for a box [xl,xu] containing a point x, a point y in the intersection 
% of [xl,xu] and [u,v] is constructed such that it is both not close to 
% x and to the boundary of [xl,xu] and its function value is estimated
% from a local quadratic model around x
%
% Input:
% x	point contained in [xl,xu]
% xl,xu	box bounds
% u,v   the point is to be generated in [u,v]
% g,G   the local quadratic model around x is given by
%       q(y) = g(1) + g(2:n+1)*(y-x)' + 0.5*g(n+2)*(y-x)*G*(y-x)
%       for a row vector y
%
% Output:
% y	point in the intersection of [xl,xu] and [u,v]
% f     corresponding estimated function value



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% snobrad.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function d = snobrad(x,y,d)
% shrinks the old trust region radii such that the point y is at 
% the boundary
% 
% Input:
% x	point around which the trust region is to be computed
% y	second point in the trust region
% d	the components are the old trust region radii



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%% snobround.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function x = snobround(x,u,v)
% a point x is projected into the interior of [u,v] and its entries are
% rounded to the nearest integer
%
% Input:
% x	vector 
% u,v	vectors of the same length as x such that u < v
%
% Output:
% x	projected and rounded version of x



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% snobscal.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [u,v,u1,v1,x,dy] = snobscal(u,v,u1,v1,x,dx,dy)
% scales the vectors u, v, u1, v1 and dy with the vector dx 
% 
% Input
% u,v,u1,v1,dx,dy   vectors of same length n, where dx has only
%                   positive entries
% x                 matrix with n columns
%
% Output:
% u,v,u1,v1,x,dy    the old quantities scaled by dx



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% snobsoft.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% file (m-script) for applying the soft optimality theorem and SNOBFIT
% to some Hock-Schittkowski problems



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%% snobsplit.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [xl,xu,x,f,nsplit,vol] = snobsplit(x,f,xl0,xu0,nspl,u,v)
% splits a box [xl0,xu0] contained in a bigger box [u,v] such that
% each of the resulting boxes contains just one point of a given set of
% points
%
% Input:
% x	the rows are a set of points
% f	f(j,:) contains the function value, its variation and possibly
%	other parameters belonging to point x(j,:)
% xl0	vector of lower bounds of the box
% xu0	vector of upper bounds of the box
% nspl	nspl(i) is the number of splits the box [xl0,xu0] has already
%	undergone along the ith coordinate
% u,v	bounds of the original problem	
%	[xl0,xu0] is contained in [u,v]
%
% Output:
% xl	xl(j,:) is the lower bound of box j
% xu	xi(j,:) is the upper bound of box j
% x      	x(j,:) is the point contained in box j
% f	f(j,:) contains the function value at x(j,:), its uncertainty
%	etc. 
% nsplit nsplit(j,i) is the number of times box j has been split in the
%	ith coordinate
% vol	vol(j) is an integer-valued logarithmic volume measure of box j



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% snobstd.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function sigma = snobstd(x,x0,f,sigmag2,sigmaG2,L)
% estimation of the standard deviation of an estimated function value
%
% Input:
% x	       point at which the standard deviation of the estimated 
%	       function value is to be estimated
% x0	       point around which the quadratic model has been estimated
% f	       vector containing the deviations of the function values
%	       of the neighbors of x0
% sigmag2      estimated variance of the local fit
% sigmaG2      estimated variance of the Hessian fit
% L	       Cholesky factor of the estimated Hessian
%
% Output:
% sigma	       estimated standard deviation



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% snobtest.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% file (m-script) for running SNOBFIT on a set of test functions 



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% snobupdt.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [xl,xu,x,f,nsplit,vol,near,d,np,t,inew,fnan,u,v] = 
% snobupdt(xl,xu,x,f,nsplit,vol,near,d,np,t,xnew,fnew,fnan,u,v,u1,v1)
% updates the box parameters when a set of new points and their function
% values are added, i.e., the boxes containing more than one point are
% split and the nearest neighbors are computed or updated
%
% Input:
% xl,xu		rows contain lower and upper bounds of the old boxes
% x		rows contain the old points
% f		f(j,:) contains the function value at x(j,:), its
%		variation and other parameters
% nsplit	nsplit(j,i) number of times box j has been split along
%		the ith coordinate
% vol		vol(j) is an integer-valued logarithmic volume measure
%		of box j
% near		near(j,:) is a vector pointing to the nearest 
%		neighbors of x(j,:)
% d		d(j) is the maximal distance between x(j,:) and one of
%		its neighbors
% np	        np(j) is the number of times the function value of 
%		x(j,:) has been measured
% t		t(j) is np(j) times the variance of the function values
%		measured for the point x(j,:)
% xnew		rows contain new points
% fnew		new function values and their variations
%		fnew(j,1) = f(xnew(j,:)), fnew(j,2) = df(xnew(j,:))
% fnan          pointer to all old points where the function value could
%               not be obtained
% u,v		box bounds
% u1,v1         box in which the new points are to be generated
%
% Output:
% xl,xu		updated version of xl,xu (including new boxes)
% x		updated version of x
% f		updated version of f
% nsplit	updated version of nsplit
% vol		updated version of vol
% near		updated version of near
% d		updated version of d
% np		updated version of np
% t		updated version of t
% inew		pointer pointing to the new boxes and boxes whose nearest
%               neighbors have changed
% fnan          possibly updated version of fnan (if a function value was
%               found for a point in the new iteration)
% u,v		possibly updated box bounds such that all new points
% 		are in the box






