%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% snobfit.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [request,xbest,fbest,acc] = snobfit(file,x,f,params,dx)
% minimization of a function over a box in R^n
%
% Input:
% file		name of file for input and output
%		if nargin < 5, the program continues a previous run and
%		reads from file.mat
%		the output is (again) stored in file.mat
% x		the rows are a set of new points entering the 
%		optimization algorithm together with their function 
%		values  
% f		matrix containing the corresponding function values
%		and their uncertainties, i.e., f(j,1) = f(x(j,:)) 
% 		and f(j,2) = df(x(j,:))
% params	structure variable defining the box [u,v] in which the
%		points are to be generated, the number nreq of 
%		points to be generated and the probability p that a 
%               point of type 4 is generated
%		params = struct('bounds',{u,v},'nreq',nreq,'p',p)
% dx		only used for the definition of a new problem (when
%		the program should continue from the values stored in
%		file.mat, the call should have only 4 input parameters!)
%	        n-vector of minimal steps, i.e., two points are 
%               considered to be different if they differ by at least 
%               dx(i) in at least one coordinate i
%
% Output:
% request	nreq x (n+3)-matrix (n = size(x,2))
%		request(j,1:n) is the jth newly generated point, 
%		request(j,n+1) is its estimated function value and
%		request(j,n+2) the corresponding estimated standard
%		deviation; request(j,n+3) indicates for which reason
%		the point request(j,1:n) has been generated 
%		request(j,n+3) = 1 best prediction
%		               = 2 explore local valley
%		               = 3 explore alternative valley
%		               = 4 explore empty region
% xbest		current best point
% fbest		current best function value (i.e. function value at 
%		xbest)
% acc		measures the accuracy of the quadratic model at the
%		best point
%		= max(abs(f(x)-q(x)), where the max is taken over the 
%		best point and its n+5 nearest neighbors and q is the 
%		quadratic model
%
% Uses the following m-files (directly or indirectly):
% minq.m and its subprograms
% rsort.m
% snob5.m
% snobacc.m
% snobbeta.m
% snobhess.m
% snobinput.m
% snoblocf.m
% snoblp.m
% snobnan.m
% snobnewb.m
% snobnn.m
% snobpoint.m
% snobrad.m
% snobround.m
% snobscal.m
% snobsplit.m
% snobstd.m
% snobupdt.m
%
function [request,xbest,fbest,acc] = snobfit(file,x,f,params,dx)
rho = 0.5*(sqrt(5)-1);	% golden section number
n = size(x,2);	% dimension of the problem
u1 = params(1).bounds;
v1 = params(2).bounds;
nreq = params(1).nreq;
p = params(1).p;
dy = 0.1*(v1-u1);% defines the vector of minimal distances between two points
                 % of type 3
if size(u1,1) > 1, u1 = u1'; end
if size(v1,1) > 1, v1 = v1'; end
if nargin > 4	% a new run is started
  if ~isempty(find(dx==0)), error('dx should not contain zero entries'), end
  if size(dx,1) > 1, dx = dx'; end
  if ~isempty(x)
    u = min([x(:,1:n);u1]);
    v = max([x(:,1:n);v1]);
  else
    u = u1;
    v = v1;
  end                     
  if size(dy,1) > 1, dy = dy'; end
  [x,f,np,t] = snobinput(x,f); % throw out duplicates among the points
                               % and compute mean function value and
                               % deviation 
  [u,v,u1,v1,x,dy] = snobscal(u,v,u1,v1,x,dx,dy); 
  % all points are scaled such that dx = ones(1,n)
  d0 = (v-u)/4;	% initial value for the diameter of the trust region
		% for local minimization from the best point
  if size(x,1) 
    [xl,xu,x,f,nsplit,vol] = snobsplit(x,f,u,v,zeros(1,n),u,v);
  end
  if size(x,1) >= n+6
    for j=1:size(x,1)
      [near(j,:),d(j)] = snobnn(x(j,:),x,n+5);
    end
    inew = 1:size(x,1);
    fnan = find(isnan(f(:,1)));
    if ~isempty(fnan)
      f = snobnan(fnan,f,near,inew);
    end
    [fbest,jbest] = min(f(:,1));
    xbest = x(jbest,:);
    [G,sigmaG2,L,beta,g00] = snobhess(x,f(:,1),xbest,fbest);
    for j=1:size(x,1)
      [y(j,:),f(j,3),sigmag2(j),g(j,:)] = snoblocf(x(j,:),x([j near(j,:)],:),f([j near(j,:)],1:2),xl(j,:),xu(j,:),G,sigmaG2,beta([j near(j,:)]),u,v);
    end
    acc = snobacc(x([jbest near(jbest,:)],:),f([jbest near(jbest,:)],1),g(jbest,:),G);
  else
    d = Inf*ones(1,size(x,1));
    fnan = [];
    near = [];
    if ~size(x,1)
      xl = [];
      xu = [];
      nsplit = [];
      vol = [];
    end
    x1 = snob5(x,u1,v1,nreq);
    request = [x1 NaN*ones(nreq,2) 5*ones(nreq,1)];
    acc = NaN;
    if ~isempty(x)
      [fbest,ibest] = min(f(:,1));
      xbest = x(ibest,:);
    else
      xbest = NaN*ones(1,n);
      fbest = Inf;
    end
    clear xnew fnew u1 v1 nreq
    save(file)
    nreq = size(request,1);
    request(:,1:n) = request(:,1:n).*(ones(nreq,1)*dx);
    if ~isempty(xbest)
      xbest = xbest.*dx;
    end
    return
  end
else
  xnew = x;
  fnew = f;
  load(file)
  nx = size(xnew,1);
  xnew = xnew./(ones(nx,1)*dx);
  u1 = u1./dx;
  v1 = v1./dx;
  % update the trust region radius
  i1 = find(request(:,n+3)==1);
  i2 = find(request(:,n+3)==2);
  if ~isempty(i1) & ~isempty(i2)
    d1 = sum((xnew-ones(size(xnew,1),1)*request(i1,1:n)).^2,2);
    [dmin,i1] = min(d1);
    d2 = sum((xnew-ones(size(xnew,1),1)*request(i2,1:n)).^2,2);
    [dmin,i2] = min(d2);
    ff1 = fnew(i1,1);
    ff2 = fnew(i2,1);
    if ff1 < min(ff2,fbest) 
      d0 = d0/rho;
    elseif fbest < min(ff1,ff2)
      d0 = d0*rho;
    end
    d0 = max(d0,1);
  end
  oldxbest = xbest;
  [xl,xu,x,f,nsplit,vol,near,d,np,t,inew,fnan,u,v] = snobupdt(xl,xu,x,f,nsplit,vol,near,d,np,t,xnew,fnew,fnan,u,v,u1,v1);
  if ~isempty(near)
    ind = find(isnan(f(:,1)));
    fnan = [fnan ind];
    if ~isempty(fnan)
      f = snobnan(fnan,f,near,inew);
    end
    [fbest,jbest] = min(f(:,1));
    xbest = x(jbest,:);
    if xbest ~= oldxbest | exist('G')~=1
      [G,sigmaG2,L,beta,g00] = snobhess(x,f(:,1),xbest,fbest);
    else
      beta = snobbeta(x,jbest,L,beta);
    end
    for k=1:length(inew)
      j = inew(k);
      [y(j,:),f(j,3),sigmag2(j),g(j,:)] = snoblocf(x(j,:),x([j near(j,:)],:),f([j near(j,:)],1:2),xl(j,:),xu(j,:),G,sigmaG2,beta([j near(j,:)]),u,v);
    end
    acc = snobacc(x([jbest near(jbest,:)],:),f([jbest near(jbest,:)],1),g(jbest,:),G);
  else
    x1 = snob5(x,u1,v1,nreq);
    request = [x1 NaN*ones(nreq,2) 5*ones(nreq,1)];
    acc = NaN;
    if ~isempty(x)
      [fbest,ibest] = min(f(:,1));
      xbest = x(ibest,:);
    else
      xbest = [];
      fbest = Inf;
    end
    clear xnew fnew u1 v1 nreq
    save(file)
    nreq = size(request,1);
    request(:,1:n) = request(:,1:n).*(ones(nreq,1)*dx);
    xbest = xbest.*dx;
    return
  end
end

sx = size(x,1);
request = [];
if sum(u1<=xbest&xbest<=v1) == n
  xl0 = max(xbest-d0,u1);
  xu0 = min(xbest+d0,v1);
  gbest = g(jbest,:);
  [z1,f1] = minq(gbest(1)-xbest*gbest(2:n+1)'+0.5*gbest(n+2)*xbest*G*xbest',gbest(2:n+1)'-gbest(n+2)*G*xbest',gbest(n+2)*G,xl0',xu0',0);
  z1 = z1';
  if isempty(find(z1==xbest-d0)) & isempty(find(z1==xbest+d0))
    d0 = snobrad(xbest,z1,d0);
  end
  xl0 = max(xbest-rho*d0,u1);
  xu0 = min(xbest+rho*d0,v1);
  [z2,f2] = minq(gbest(1)-xbest*gbest(2:n+1)'+0.5*gbest(n+2)*xbest*G*xbest',gbest(2:n+1)'-gbest(n+2)*G*xbest',gbest(n+2)*G,xl0',xu0',0);
  z2 = z2';
  z1 = snobround(z1,u1,v1);
  z2 = snobround(z2,u1,v1);
  zz = ones(size(x,1),1)*z1;
  j = find(sum(xl<=zz&zz<=xu,2)==n);
  if min(max(abs(x(j,:)-ones(length(j),1)*z1),[],2)) >= 1
    if length(j) > 1
      [mvol,j1] = min(vol(j));
      j = j(j1);
    end
    sreq = size(request,1);
    if max(xu(j,:)-xl(j,:)) > 1 & (~sreq | min(max(abs(ones(sreq,1)*z1-request(:,1:n)),[],2))>=1)
      sigma = snobstd(z1,xbest,f([jbest near(jbest,:)],2),sigmag2(jbest),sigmaG2,L);
      request = [request; z1 f1 sigma 1];
    end
  end
  zz = ones(size(x,1),1)*z2;
  j = find(sum(xl<=zz&zz<=xu,2)==n);
  if min(max(abs(x(j,:)-ones(length(j),1)*z2),[],2)) >= 1
    if length(j) > 1
      [mvol,j1] = min(vol(j));
      j = j(j1);
    end
    sreq = size(request,1);
    if max(xu(j,:)-xl(j,:)) > 1 & (~sreq | min(max(abs(ones(sreq,1)*z2-request(:,1:n)),[],2))>=1)
      sigma = snobstd(z2,xbest,f([jbest near(jbest,:)],2),sigmag2(jbest),sigmaG2,L);
      request = [request; z2 f2 sigma 2];
    end
  end
else
  ind = find(sum(ones(sx,1)*u1<=x&x<=ones(sx,1)*v1)==n);
  if ~isempty(ind)
    [fbes,jbes] = min(f(ind,1));
    jbes = ind(jbes);
    xbes = x(jbes,:);
    gbes = g(jbes,:);
    d1 = (v1-u1)/4;
    xl0 = max(xbes-d1,u1);
    xu0 = min(xbes+d1,v1);
    [z1,f1] = minq(gbes(1)-xbes*gbes(2:n+1)'+0.5*gbes(n+2)*xbes*G*xbes',gbes(2:n+1)'-gbes(n+2)*G*xbes',gbes(n+2)*G,xl0',xu0',0);
    z1 = z1';
    if isempty(find(z1==x(jbes,:)-d0)) & isempty(find(z1==x(jbes,:)+d0))
      d1 = snobrad(xbes,z1,d0);
    end
    xl0 = max(xbes-rho*d1,u1);
    xu0 = min(xbes+rho*d1,v1);
    [z2,f2] = minq(gbes(1)-xbes*gbes(2:n+1)'+0.5*gbes(n+2)*xbes*G*xbes',gbes(2:n+1)'-gbes(n+2)*G*xbes',gbes(n+2)*G,xl0',xu0',0);
    z2 = z2';
    z1 = snobround(z1,u1,v1);
    z2 = snobround(z2,u1,v1);
    zz = ones(size(x,1),1)*z1;
    j = find(sum(xl<=zz&zz<=xu,2)==n);
    if min(max(abs(x(j,:)-ones(length(j),1)*z1),[],2)) >= 1
      if length(j) > 1
        [mvol,j1] = min(vol(j));
        j = j(j1);
      end
      sreq = size(request,1);
      if max(xu(j,:)-xl(j,:)) > 1 & (~sreq | min(max(abs(ones(sreq,1)*z1-request(:,1:n)),[],2))>=1)
        sigma = snobstd(z1,xbes,f([jbes near(jbes,:)],2),sigmag2(jbes),sigmaG2,L);
        request = [request; z1 f1 sigma 1];
      end
    end
    zz = ones(size(x,1),1)*z2;
    j = find(sum(xl<=zz&zz<=xu,2)==n);
    if min(max(abs(x(j,:)-ones(length(j),1)*z2),[],2)) >= 1
      if length(j) > 1
        [mvol,j1] = min(vol(j));
        j = j(j1);
      end
      sreq = size(request,1);
      if max(xu(j,:)-xl(j,:)) > 1 & (~sreq | min(max(abs(ones(sreq,1)*z2-request(:,1:n)),[],2))>=1)
        sigma = snobstd(z2,xbes,f([jbes near(jbes,:)],2),sigmag2(jbes),sigmaG2,L);
        request = [request; z2 f2 sigma 2];
      end
    end
  end  
end

isplit = [];
if size(request,1) < nreq 
  ind = find(sum(xl<=ones(sx,1)*v1&xu>=ones(sx,1)*u1,2)==n);
  globloc = nreq - size(request,1);
  glob1 = globloc*p;
  glob2 = floor(glob1);
  if rand < glob1 - glob2;
    glob = glob2 + 1;
  else
    glob = glob2;
  end
  loc = globloc - glob;
  if loc
    [local,nlocal] = snoblp(f(:,1),near,ind);
    [fsort,k] = sort(min(f(local,1),f(local,3)));
    j = 1;
    sreq = size(request,1);
    while sreq < nreq-glob & j <= length(local)
      l = local(k(j));
      y1 = min(max(y(l,:),u1),v1);
      if max(abs(y(l,:)-x(l,:)))>=1 & (~sreq | min(max(abs(request(:,1:n)-ones(sreq,1)*y1),[],2))>=1) & min(max(abs(x-ones(sx,1)*y1),[],2))>=1 
        if sum(y1==y(l,:)) < n
          f1 = g(l,1)+g(l,2:n+1)*(y1-x(l,:))'+0.5*g(l,n+2)*(y1-x(l,:))*G*(y1-x(l,:))';
        else
          f1 = f(l,3);
        end
        sigma = snobstd(y1,x(l,:),f([l near(l,:)],2),sigmag2(l),sigmaG2,L); 
        isplit = [isplit l];
        request = [request; y1 f1 sigma 3];
      end
      sreq = size(request,1);
      j = j + 1;      
    end
    if sreq < nreq-glob
      [fsort,k] = sort(min(f(nlocal,1),f(nlocal,3)));
    end
    j = 1;
    while sreq < nreq-glob & j<= length(nlocal) 
      l = nlocal(k(j));
      y1 = min(max(y(l,:),u1),v1);
      if max(abs(y(l,:)-x(l,:)))>=1 & (~sreq | min(max(abs(request(:,1:n)-ones(sreq,1)*y1)-ones(sreq,1)*dy,[],2))>=0) & min(max(abs(x-ones(sx,1)*y1),[],2))>=1 
        if sum(y1==y(l,:)) < n
          f1 = g(l,1)+g(l,2:n+1)*(y1-x(l,:))'+0.5*g(l,n+2)*(y1-x(l,:))*G*(y1-x(l,:))';
        else
          f1 = f(l,3);
        end
        sigma = snobstd(y1,x(l,:),f([l near(l,:)],2),sigmag2(l),sigmaG2,L);
        isplit = [isplit l]; 
        request = [request; y1 f1 sigma 3];
      end
      sreq = size(request,1);
      j = j + 1;
    end
  end
end
for j=1:length(isplit)
  jj = find(ind==isplit(j));
  ind(jj) = [];
end
if size(request,1) < nreq & ~isempty(ind)
  [minvol,k] = min(vol(ind));
  m = 0;
  sreq = size(request,1);
  while sreq < nreq & minvol+m <= max(vol(ind))
    k = find(vol(ind)==minvol+m);
    if ~isempty(k)
      k = ind(k);
      [fsort,j] = sort(f(k,1));
      k = k(j);
      j = 1;
      while sreq < nreq & j <= length(k)
        l = k(j);
        [y1,f1] = snobpoint(x(l,:),xl(l,:),xu(l,:),g(l,:),G,u1,v1);
        if ~sreq | min(max(abs(request(:,1:n)-ones(sreq,1)*y1),[],2))>=1
          sigma = snobstd(y1,x(l,:),f([l near(l,:)],2),sigmag2(l),sigmaG2,L);  
          request = [request; y1 f1 sigma 4];
        end
        sreq = size(request,1);
        j = j + 1;
      end
    end
    m = m + 1;
  end
end
if size(request,1) < nreq
  x1 = snob5([x; request(:,1:n)],u1,v1,nreq-size(request,1));
  nx = size(x,1);
  for j=1:size(x1,1)
    i=find(~sum(max(xl-ones(nx,1)*x1(j,:),0)+max(ones(nx,1)*x1(j,:)-xu,0),2));
    if length(i) > 1
      [minv,i1] = min(vol(i));
      i = i(i1);
    end
    f1 = g(i,1) + (x1(j,:)-x(i,:))*g(i,2:n+1)' + 0.5*g(i,n+2)*(x1(j,:)-x(i,:))*G*(x1(j,:)-x(i,:))';
    sigma = snobstd(x1(j,:),x(i,:),f([i near(i,:)],2),sigmag2(i),sigmaG2,L);
    request = [request; x1(j,:) f1 sigma 5];
  end
end
clear xnew fnew u1 v1 nreq p
save(file)
nreq = size(request,1);
request(:,1:n) = request(:,1:n).*(ones(nreq,1)*dx);
xbest = xbest.*dx;


