%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% snobhess.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [G,sigmaG2,L,beta,g] = snobhess(x,f,xbest,fbest)
% computes a Hessian fit around the point xbest
% f_k-fbest = g'*s_k+0.5*s_k'*G*s_k+eps_k norm(L*s_k)^2, where L is 
% defined as below, s_k = x_k-xbest and sum(eps_k^2) is minimized
%
% Input:
% x		rows are the set of points to be considered
% f		corresponding function values 
% xbest		best point (among the x(j,:))
% fbest		its function value		
%
% Output:
% G		estimate for the Hessian
% sigmaG2	variance of the errors
% L		Cholesky factor of inv(S'*S), where 
%               S = x - ones(K,1)*xbest, K = size(x,1), where the row 
%               with the xbest has been canceled  
% beta		beta(j) = norm(L*(x(j,:)-xbest)')^2
% g		estimate for the gradient
%
function [G,sigmaG2,L,beta,g] = snobhess(x,f,xbest,fbest)
K = size(x,1);
n = size(x,2);	% dimension of the problem
ind = find(sum(x==ones(K,1)*xbest,2)==n);
if ~isempty(ind)
  x(ind,:) = [];
  f(ind,:) = [];
  K = K - 1;	% number of points ~= xbest
end
S = x - ones(K,1)*xbest;
R = triu(qr(S,0));
R = R(1:n,:);
L = inv(R)';
beta = sum((L*S').^2,1)';
b = (f-fbest)./beta;
A = [x - ones(K,1)*xbest 0.5*(x - ones(K,1)*xbest).^2];
for i=1:n-1
  B = (x(:,i)-xbest(i))*ones(1,n-i);
  A = [A B.*(x(:,i+1:n)-ones(K,1)*xbest(i+1:n))];
end
A = A./(beta*ones(1,size(A,2)));
y = A\b;
N = length(find(y));
for i=1:n
  G(i,i) = y(n+i);
end
l = 2*n+1;
for i = 1:n-1
  for j=i+1:n
    G(i,j) = y(l);
    G(j,i) = y(l);
  end
  l = l + 1;
end
if K > N
  sigmaG2 = sum((A*y-b).^2)/(K-N);
else
  sigmaG2 = sum((A*y-b).^2);
end
beta = [beta(1:ind-1); 0; beta(ind:length(beta))];
g = y(1:n);
