%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% snoblocf.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [y,f,sigmag2,g] = snoblocf(x0,x,f,xl,xu,G,sigmaG2,beta,u,v)
% computes a local fit around the point x0
% f_k = f + g'*s_k+0.5*gamma*s_k'*G*s_k + eps_k*sqrt(Deltaf_k^2
% +sigmaG2*beta_k)
%
% Input:
% x0		point around which the fit is to be computed
% x		the rows of x are the point x0 (first row) and its
% 		safeguarded nearest neighbors
% f		the corresponding function values and their 
%		uncertainties, i.e., f(j,1) = f(x(j,:)), 
%		f(j,2) = df(x(j,:))
% xl,xu		lower and upper bounds of the box containing x0
% G		estimated Hessian (from snobhess)
% sigmaG2	variance of the errors of the Hessian fit (from 
%		snobhess) 
% beta		beta(j) = norm(L*(x(j,:)-xbest)')^2 (L matrix from 
%		snobhess)
% u,v           box bounds
%
% Output:
% y		best point found by optimizing the local model over
%		the box [xl0,xu0]
% f 		estimated best function value
% sigmag2	variance of the fit
% g		g(1) estimated constant term (f), g(2:n+1) estimated 
%		gradient, g(n+2) estimated coefficient gamma in front
%		of the quadratic term
%
function [y,f,sigmag2,g] = snoblocf(x0,x,f,xl,xu,G,sigmaG2,beta,u,v)
K = size(x,1);
n = size(x,2);
b = f(:,1)./sqrt(f(:,2).^2+sigmaG2*beta.^2);
S = x - ones(K,1)*x0;
A = [ones(K,1) S];
for k=1:K
  A(k,n+2) = 0.5*S(k,:)*G*S(k,:)';
end
A = A./sqrt((f(:,2).^2+sigmaG2*beta.^2)*ones(1,size(A,2)));
g = A\b;
N = length(find(g));
sigmag2 = sum((A*g-b).^2)/(K-N);
w = 0.05*(xu-xl);
xl0 = xl + w;
xu0 = xu - w;
dmax = max(xu-xl);
i=find(xu-xl<=0.05*dmax);
if ~isempty(i)
  xl0(i) = x0(i);
  xu0(i) = xl0(i);
end
[y,f] = minq(g(1)-x0*g(2:n+1)+0.5*g(n+2)*x0*G*x0',g(2:n+1)-g(n+2)*G*x0',g(n+2)*G,xl0',xu0',0);
y = y';
if isempty(find(abs(x0-y)>=w))
  ind = find(max(xu0-y,y-xl0)>=w);
  if isempty(ind),error('shifting not possible'), end
  dmax = max(abs(x0(ind)-y(ind))./w(ind));
  i = find(abs(x0(ind)-y(ind))./w(ind)==dmax);
  ind = ind(i);
  [dmax,i] = max((xu(ind)-xl(ind))./(v(ind)-u(ind)));
  i = ind(i);
  if y(i) < x0(i) & x0(i) - w(i) >= xl0(i)
    y(i) = x0(i) - w(i);
  elseif y(i) > x0(i) & x0(i) + w(i) <= xu0(i)
    y(i) = x0(i) + w(i);
  else
    if x0(i) + w(i) <= xu0(i)
      y(i) = x0(i) + w(i);
    else
      y(i) = x0(i) - w(i);
    end
  end
  y = snobround(y,xl0,xu0);
  f1 = g(1) + (y-x0)*g(2:n+1) + 0.5*g(n+2)*(y-x0)*G*(y-x0)';
end
g = g';
