%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% snobsoft.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% file (m-script) for applying the soft optimality theorem and SNOBFIT
% to some Hock-Schittkowski problems
clear; clear global; clear mex
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% beginning of data to be adapted
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

prob = 18;             % problem number
fun = 'hsf18';         % name of objective function
Fun = 'hsF18';         % name of constraint function
file = 'hs';           % the intermediate data (after each call to SNOBFIT)
                       % are stored in hs.mat
[u,v,F1,F2] = hsbounds(prob); 
                       % default bounds for variables and constraints
n = length(u);         % dimension of the problem
ncall = 4000;          % limit on the number of function calls
npoint = n+6;          % the standard starting point and npoint-1
                       % randomly generated points are used as input 
                       % for the initial call to SNOBFIT
nreq = n+6;            % number of points to be generated in
                       % continuation calls to SNOBFIT
ftr = -0.9;            % merit function value to be reached (in ]-1,3[)
dx = (v-u)'*1.e-4;     % resolution vector
p = 0.1;               % fraction of points of class 4 among the points
                       % of class 3 and 4
m = length(F1);        % number of constraints
sigma = 0.1*ones(m,1); % choice of sigma in the merit function
Delta = 10;            % parameter in the merit function
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% end of data to be adapted
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

params = struct('bounds',{u,v},'nreq',nreq,'p',p); % input structure
x0 = hsstart(prob); % Hock-Schittkowski standard starting point
% generation of npoint-1 random points in [u,v]
x = rand(npoint-1,n);
x = x*diag(v-u)+ones(npoint-1,1)*u';
x = [x0';x];
% computation of the objective function values and constraints for the 
% starting points
f0 = Inf; % initialization of f0
for j=1:npoint
  f(j) = feval(fun,x(j,:));
  FF = feval(Fun,x(j,:));
  if size(FF,1) > 1
    F(j,:) = FF';
  else
    F(j,:) = FF';
  end
  if (sum(F1'<=F(j,:)&F(j,:)<=F2')==n)
    f0 = min(f0,f(j)); % f0 is the smallest function value among the
                       % feasible ones of the initial points
  end
end
if isinf(f0) 
  f0 = 2*max(f)-min(f);
end
% computation of the merit function for the starting points
for j=1:npoint
  delta = 0;
  for i=1:m
    if (F(j,i) < F1(i))
      delta = delta + (F1(i)-F(j,i))^2/sigma(i)^2;
    elseif (F(j,i) > F2(i))
      delta = delta + (F(j,i)-F2(i))^2/sigma(i)^2;
    end
  end
  % computation of the merit function
  fm(j,1) = (f(j)-f0)/(Delta+abs(f(j)-f0)) + 2*delta/(1+delta);
end
fm(:,2) = sqrt(eps)*ones(npoint,1); % set uncertainty of the merit 
                                    % function
ncall0 = npoint; % function call counter
while ncall0 < ncall  % repeat until the limit on function calls is 
                      % reached
  if ncall0 == npoint % initial call
    [request,xbest,fbest] = snobfit(file,x,fm,params,dx);
  else                % continuation call
    [request,xbest,fbest] = snobfit(file,x,fm,params);
  end
  req = request(:,[1:n+1 n+3]) % shows the suggested points, estimated
                               % merit function values and class (rowwise)
  clear x; clear fm
  % compute merit function at the suggested points
  for j=1:size(request,1)
    x(j,:) = request(j,1:n);
    ff = feval(fun,x(j,:));
    FF = feval(Fun,x(j,:));
    delta = 0;
    for i=1:m
      if (FF(i)<F1(i))
        delta = delta + (F1(i)-FF(i))^2/sigma(i)^2;
      elseif (FF(i)>F2(i))
        delta = delta + (FF(i)-F2(i))^2/sigma(i)^2;
      end
    end
    fm(j,1) = (ff-f0)/(Delta+abs(ff-f0)) + 2*delta/(1+delta);
  end
  fm(:,2) = sqrt(eps)*ones(size(request,1),1); 
  ncall0 = ncall0 + size(request,1); % update function call counter
  [fbestn,jbest] = min(fm(:,1)); % best of the new merit function values
  % if a better function value has been found, update fbest
  if fbestn < fbest
    fbest = fbestn;
    xbest = x(jbest,:);
    xbest,fbest % display current best point and merit function value if
                % it has changed
  end
  if fbest <= ftr break, end % check stopping criterion (reaching the 
                             % merit function value ftr)
end
xbest,fbest % display best point and merit function value
ncall0  % displaypin number of function values used
